
#include <ctype.h>
#include <vector>
#include "image/image.h"

#include "captcha.h"
#include "rand64.h"

namespace soramimi {

	void captcha(char const *string, int length, soramimi::image::Image const *digits, int font, soramimi::image::Image *image, int *image_w_p, int *image_h_p)
	{
		int w = 0;
		int h = 0;

		int margin = 10;

		int image_w = 32 * length + margin * 2;
		int image_h = 48 + margin * 2;
		image->clear(soramimi::image::PT_YA);
		image->fill(0, 0, image_w, image_h, soramimi::image::PixelRGB(0, 0, 0, 255));

		int prenoise = image_w * image_h / 400;
		int postnoise = image_w * image_h / 5;

		for (int i = 0; i < length; i++) {
			soramimi::image::PixelY tmp[16 * 32];
			int c = string[i];
			int n = -1;
			if (c >= '0' && c <= '9') {
				n = c - '0';
			} else if (c >= 'A' && c <= 'Z') {
				n = c - 'A' + 10;
			} else if (c >= 'a' && c <= 'z') {
				n = c - 'a' + 10 + 26;
			}
			if (n >= 0) {
				digits->render(n * 16, font * 32, 16, 32, &tmp[0]);
				int x = random64() % 16;
				int y = random64() % 16;
				image->put(i * 32 + margin + x, margin + y, &tmp[0], 16, 0, 0, 16, 32);
			}
		}

		image->zebra(0, 0, image_w, image_h, 2, 1);

		{
			int n = image_w * image_h;
			std::vector<soramimi::image::PixelY> tmp(n);
			image->render(0, 0, image_w, image_h, &tmp[0]);
			for (int i = 0; i < prenoise; i++) {
				int j;
				j = random64() % n;
				tmp[j] = soramimi::image::PixelY(255, 255);
			}
			image->put(0, 0, &tmp[0], image_w, 0, 0, image_w, image_h);
		}

		image->zebra(0, 0, image_w, image_h, 2, 10);

		image->invert(0, 0, image_w, image_h);

		{
			double freq = 20;
			double span = 1;
			double angle = random64() / (double)0xffffffffffffffffULL * 360;
			double ox = random64() / (double)0xffffffffffffffffULL * image_w;
			double oy = random64() / (double)0xffffffffffffffffULL * image_h / 2;
			image->wave(0, 0, image_w, image_h, ox, oy, freq, span, angle);
		}

		{
			int n = image_w * image_h;
			std::vector<soramimi::image::PixelY> tmp(n);
			image->render(0, 0, image_w, image_h, &tmp[0]);
			for (int i = 0; i < postnoise; i++) {
				int j;
				j = random64() % n;
				tmp[j] = soramimi::image::PixelY(random64() % 256, 255);
			}
			image->put(0, 0, &tmp[0], image_w, 0, 0, image_w, image_h);
		}

		*image_w_p = image_w;
		*image_h_p = image_h;
	}

}
